"""
lodes_star

For scraping Longitudinal Origin-Destination Employment Statisics (LODES) data from the US Census as well as fetching Census Block GIS shapefiles.
"""

__version__ = "0.2.5"
__author__ = 'Nicholas Fournier'
